@@ -1197,20 +1197,28 @@ using CPANPLUS underneath.
 
 For detailed documentation, refer to L<CPAN>.
 
-=head1 AUTHORS
+=head1 BUG REPORTS
 
-This module by
-Jos Boumans E<lt>kane@cpan.orgE<gt>.
+Please report bugs or other issues to E<lt>bug-cpanplus@rt.cpan.org<gt>.
+
+=head1 AUTHOR
+
+This module by Jos Boumans E<lt>kane@cpan.orgE<gt>.
 
 =head1 COPYRIGHT
 
-The CPANPLUS interface (of which this module is a part of) is
-copyright (c) 2001-2004 Jos Boumans E<lt>kane@cpan.orgE<gt>.
-All rights reserved.
+The CPAN++ interface (of which this module is a part of) is copyright (c) 
+2001 - 2007, Jos Boumans E<lt>kane@cpan.orgE<gt>. All rights reserved.
+
+This library is free software; you may redistribute and/or modify it 
+under the same terms as Perl itself.
+
+=head1 SEE ALSO
+
+L<CPANPLUS::Configure>, L<CPANPLUS::Module>, L<CPANPLUS::Module::Author>
+
+=cut
 
-This library is free software;
-you may redistribute and/or modify it under the same
-terms as Perl itself.
 
 =head1 SEE ALSO
 
